/*
 * PHOSPHOR_PROGRAMMER
 *
 * Created: March 2019
 * Author: Joris Wegner (joriswegner.de)
 *
 *
 * HARDWARE: Connect an analog video source to the input (PAL/NTSC) and some kind of monitor to the output.
 * A CRT monitor works best. Projectors and other devices might need time base correction first.
 * Plug in a 9V (+)- power supply and the USB cble coming from your computer.
 * 
 * Use this sketch together with the supplied Arduino sketch to find glitchy patches you like.
 * Upload the PHOSPHOR_PROGRAMMER.ino sketch to the Arduino, start this processing sketch and start experimenting.
 * You probably have to adjust the port. This can be done in line 99. 
 *
 *
 * USE:    Use the sliders to roughly adjust the brightness of the LEDs controlling the video circuit.
 *         The buttons on the right side of the sliders can be used for fine adjustment.
 *         0 = effect is off, 4095 = full effect (probably resulting in synchronisation loss)
 *         Exception: The topmost slider adjusts the amplification of the video enhancer.
 *         0 = full amplification, 4095 = black screen
 *         The toggle switches under some of the sliders allow for further variation.
 *
 *         PLEASE NOTE: The photoresistors I used have a highly exponential resistance curve. That means that
 *         the difference between the values 0 and 1 is much greater than between the values 4094 and 4095.
 *
 * RANDOM: Randomize values and switch positions.
 * INIT:   Set all values to zero and the toggle switches to their inital state.
 * WRITE:  Print the current slider and switch values. This is a preset that can be copied/pasted into a matrix in the
 *         PHOSPHOR_MIDI arduino sketch for using it later with MIDI controls.
 * 
 * USE THIS DEVICE AT YOUR OWN RISK. Using the original software to create and use patches as well as the recommended
 * parts, the unit should use much less current then a usb port can deliver. The video circuitry should be isolated 
 * well enough from the controller circuit that is connected to the computer. However, I am not responsible for any damage
 * that could occur while using this device.
 *
 * This sketch throws a lot of exceptions on startup, it should work nonetheless, sorry for that.
 *
 */ 


// import the processing serial library and the GUI library
import processing.serial.*;
import controlP5.*;
import cc.arduino.*;
Arduino arduino;
Serial port;
ControlP5 cp5;
PImage img;

int barColor = color(255, 255, 255);
int brght;                          // Background color
int vDist = 50;                     // Vertical Distance between elements
int sXpos = 50;                     // X-Position of sliders
int sWidth = 400;                   // Width of sliders
int sHeight = 15;                   // Height of sliders
int bWidth = 40;                    // Width of buttons
int bOffM = sXpos + sWidth + 30;    // Offset of minus buttons
int bOffP = bOffM + bWidth + 5;     // Offset of plus buttons
int mBWidth = 40;                   // medium button width
int lBWidth = 80;                   // Width of large buttons
int lBOff = lBWidth + 5;            // Offset of large buttons

// This is needed to make the sliders show only ints, not floats
int Pot1;
int Pot2;
int Pot3;
int Pot4;
int Pot5;
int LDR;

// Initialize ALL the variables of the interface
// These are named after their physical counterparts
int pot1Val = 0;
int sw1aVal = 0;
int sw1bVal = 0;
int sw2aVal = 0;
int sw2bVal = 0;
int sw3aVal = 0;
int sw3bVal = 0;
int pot4Val = 0;
int pot5aVal = 0;
int pot5bVal = 0;
int ldrVal = 0;

int sw1sw2val = 0;          // Which one of the four buttons below slider 2 is active?

boolean debug = false;      // Change to true to see what the program is doing



void setup() {

  // This prints a list of all available serial ports (maybe before a lot of exceptions are thrown).
  println(Arduino.list());
  println(" ");
  // This chooses the serial port with the Arduino hopefully attached to it.  
  // If needed, change the 0 to the position of the port the Arduino is using.
  String portName = Serial.list()[3];
  port = new Serial(this, portName, 115200);

  size(620, 800);
  img = loadImage("logofull.png");
  tint(255,255,255,55);
  noStroke();
  cp5 = new ControlP5(this);

  brght = 0;
  cp5.setColorForeground(color(125));
  cp5.setColorBackground(color(85));
  //cp5.setFont(font);
  cp5.setColorActive(color(125));

  // All the interface elements are created

  cp5.addSlider("Pot1")
    .setPosition(sXpos, vDist)
    .setSize(sWidth, sHeight)
    .setRange(0, 4095)
    .setValue(0)
    ;
  cp5.addButton("Pot1-")
    .setPosition(bOffM, vDist)
    .setSize(bWidth, sHeight)
    ;
  cp5.addButton("Pot1+")
    .setPosition(bOffP, vDist)
    .setSize(bWidth, sHeight)
    ;


  cp5.addSlider("Pot2")
    .setPosition(sXpos, vDist*3)
    .setSize(sWidth, sHeight)
    .setRange(0, 4095)
    .setValue(0)
    ;
  cp5.addButton("Pot2-")
    .setPosition(bOffM, vDist*3)
    .setSize(bWidth, sHeight)
    ;
  cp5.addButton("Pot2+")
    .setPosition(bOffP, vDist*3)
    .setSize(bWidth, sHeight)
    ;
  cp5.addToggle("Sw1a")
    .setPosition(sXpos, vDist*3.5)
    .setSize(mBWidth, sHeight)
    .setValue(true)
    ;
  cp5.addToggle("Sw1b")
    .setPosition(sXpos+mBWidth, vDist*3.5)
    .setSize(mBWidth, sHeight)
    ;
  cp5.addToggle("Sw2a")
    .setPosition(sXpos+mBWidth*2, vDist*3.5)
    .setSize(mBWidth, sHeight)
    .setValue(false)
    ;
  cp5.addToggle("Sw2b")
    .setPosition(sXpos+mBWidth*3, vDist*3.5)
    .setSize(mBWidth, sHeight)
    ;
  cp5.getController("Sw1a").getCaptionLabel().align(CENTER, CENTER).setPaddingX(0);
  cp5.getController("Sw1b").getCaptionLabel().align(CENTER, CENTER).setPaddingX(0);
  cp5.getController("Sw2a").getCaptionLabel().align(CENTER, CENTER).setPaddingX(0);
  cp5.getController("Sw2b").getCaptionLabel().align(CENTER, CENTER).setPaddingX(0);


  cp5.addSlider("Pot3")
    .setPosition(sXpos, vDist*5)
    .setSize(sWidth, sHeight)
    .setRange(0, 4095)
    .setValue(0)
    ;
  cp5.addButton("Pot3-")
    .setPosition(bOffM, vDist*5)
    .setSize(bWidth, sHeight)
    ;
  cp5.addButton("Pot3+")
    .setPosition(bOffP, vDist*5)
    .setSize(bWidth, sHeight)
    ;
  cp5.addToggle("Sw3")
    .setPosition(sXpos, vDist*5.5)
    .setSize(mBWidth*2, sHeight)
    .setValue(true)
    .setMode(ControlP5.SWITCH)
    ;
  cp5.getController("Sw3").getCaptionLabel().align(CENTER, CENTER).setPaddingX(0);


  cp5.addSlider("Pot4")
    .setPosition(sXpos, vDist*7)
    .setSize(sWidth, sHeight)
    .setRange(0, 4095)
    .setValue(0)
    ;
  cp5.addButton("Pot4-")
    .setPosition(bOffM, vDist*7)
    .setSize(bWidth, sHeight)
    ;
  cp5.addButton("Pot4+")
    .setPosition(bOffP, vDist*7)
    .setSize(bWidth, sHeight)
    ;


  cp5.addSlider("Pot5")
    .setPosition(sXpos, vDist*9)
    .setSize(sWidth, sHeight)
    .setRange(0, 4095)
    .setValue(0)
    ;
  cp5.addButton("Pot5-")
    .setPosition(bOffM, vDist*9)
    .setSize(bWidth, sHeight)
    ;
  cp5.addButton("Pot5+")
    .setPosition(bOffP, vDist*9)
    .setSize(bWidth, sHeight)
    ;
  cp5.addToggle("Sw5")
    .setPosition(sXpos, vDist*9.5)
    .setSize(mBWidth*2, sHeight)
    .setValue(true)
    .setMode(ControlP5.SWITCH)
    ;
  cp5.getController("Sw5").getCaptionLabel().align(CENTER, CENTER).setPaddingX(0);


  cp5.addSlider("LDR")
    .setPosition(sXpos, vDist*11)
    .setSize(sWidth, sHeight)
    .setRange(0, 4095)
    .setValue(0)
    ;
  cp5.addButton("LDR-")
    .setPosition(bOffM, vDist*11)
    .setSize(bWidth, sHeight)
    ;
  cp5.addButton("LDR+")
    .setPosition(bOffP, vDist*11)
    .setSize(bWidth, sHeight)
    ;


  cp5.addButton("Write Preset")
    .setPosition(sXpos, vDist*14)
    .setSize(lBWidth, sHeight)
    ;

  cp5.addButton("Init")
    .setPosition(sXpos + lBOff, vDist*14)
    .setSize(lBWidth, sHeight)
    ;

  cp5.addButton("Random")
    .setPosition(sXpos + lBOff*2, vDist*14)
    .setSize(lBWidth, sHeight)
    ;
    
  delay(5000);
}


// Loop to draw the background and call the update function
void draw() {      
  background(brght);
  float logowidth = 100;
  image(img, width-147, height-150, logowidth, logowidth*1.16);
 //image(img, width-157, height-150, logowidth, logowidth*1.16);
  update();
}


// If an element is used, something happens in here accordingly
void controlEvent(ControlEvent theEvent) {
  if (theEvent.isController()) {

    if (debug)print("control event from : "+theEvent.getController().getName());
    if (debug)println(", value : "+theEvent.getController().getValue());


    if (theEvent.getController().getName()=="Pot1") {
      pot1Val = int(theEvent.getController().getValue());       
      if (debug) println("pot1Val " + pot1Val);
    }
    if (theEvent.getController().getName()=="Pot1+") {
      int val1 = int(cp5.getController("Pot1").getValue()) + 1;
      cp5.getController("Pot1").setValue(val1);
    }
    if (theEvent.getController().getName()=="Pot1-") {
      int val1 = int(cp5.getController("Pot1").getValue()) - 1;
      cp5.getController("Pot1").setValue(val1);
    }


    if (theEvent.getController().getName()=="Pot2") {
      int val2 = int(theEvent.getController().getValue());
      if (sw1sw2val == 0) {
        sw1aVal = val2;
        sw1bVal = 0;
        sw2aVal = 0;
        sw2bVal = 0;
        if (debug) println("sw1a " + val2);
      }
      if (sw1sw2val == 1) {
        sw1aVal = 0;
        sw1bVal = val2;
        sw2aVal = 0;
        sw2bVal = 0;
        if (debug) println("sw1b " + val2);
      }
      if (sw1sw2val == 2) {
        sw1aVal = 0;
        sw1bVal = 0;
        sw2aVal = val2;
        sw2bVal = 0;
        if (debug) println("sw2a " + val2);
      }
      if (sw1sw2val == 3) {
        sw1aVal = 0;
        sw1bVal = 0;
        sw2aVal = 0;
        sw2bVal = val2;
        if (debug) println("sw2b " + val2);
      }
    }
    if (theEvent.getController().getName()=="Pot2+") {
      int val2 = int(cp5.getController("Pot2").getValue()) + 1;
      cp5.getController("Pot2").setValue(val2);
    }
    if (theEvent.getController().getName()=="Pot2-") {
      int val2 = int(cp5.getController("Pot2").getValue()) - 1;
      cp5.getController("Pot2").setValue(val2);
    }
    if (theEvent.getController().getName()=="Sw1a") {
      if (cp5.getController("Sw1a").getValue() == 1) {
        cp5.getController("Sw2a").setValue(0);
        cp5.getController("Sw1b").setValue(0); 
        cp5.getController("Sw2b").setValue(0); 
        sw1sw2val = 0;
        int val2 = int(cp5.getController("Pot2").getValue());
        sw1aVal = val2;
        sw1bVal = 0;
        sw2aVal = 0;
        sw2bVal = 0;
        if (debug)println(sw1sw2val);
      }
    }
    if (theEvent.getController().getName()=="Sw2a") {
      if (cp5.getController("Sw2a").getValue() == 1) {
        cp5.getController("Sw1a").setValue(0);
        cp5.getController("Sw1b").setValue(0); 
        cp5.getController("Sw2b").setValue(0); 
        sw1sw2val = 2;
        int val2 = int(cp5.getController("Pot2").getValue());
        sw1aVal = 0;
        sw1bVal = 0;
        sw2aVal = val2;
        sw2bVal = 0;
        if (debug)println(sw1sw2val);
      }
    }
    if (theEvent.getController().getName()=="Sw1b") {
      if (cp5.getController("Sw1b").getValue() == 1) {
        cp5.getController("Sw1a").setValue(0);
        cp5.getController("Sw2a").setValue(0); 
        cp5.getController("Sw2b").setValue(0); 
        int val2 = int(cp5.getController("Pot2").getValue());
        sw1aVal = 0;
        sw1bVal = val2;
        sw2aVal = 0;
        sw2bVal = 0;
        if (debug)println(sw1sw2val);
      }
    }
    if (theEvent.getController().getName()=="Sw2b") {
      if (cp5.getController("Sw2b").getValue() == 1) {
        cp5.getController("Sw1a").setValue(0);
        cp5.getController("Sw2a").setValue(0); 
        cp5.getController("Sw1b").setValue(0); 
        sw1sw2val = 3;
        int val2 = int(cp5.getController("Pot2").getValue());
        sw1aVal = 0;
        sw1bVal = 0;
        sw2aVal = 0;
        sw2bVal = val2;
        if (debug)println(sw1sw2val);
      }
    }


    if (theEvent.getController().getName()=="Pot3") {
      int val3 = int(theEvent.getController().getValue());
      if (cp5.getController("Sw3").getValue() == 1) {
        sw3aVal = val3;
        sw3bVal = 0;
        if (debug) println("sw3aVal " + val3);
      }
      if (cp5.getController("Sw3").getValue() == 0) {
        sw3aVal = 0;
        sw3bVal = val3;
        if (debug) println("sw3bVal " + val3);
      }
    }
    if (theEvent.getController().getName()=="Pot3+") {
      int val3 = int(cp5.getController("Pot3").getValue()) + 1;
      cp5.getController("Pot3").setValue(val3);
    }
    if (theEvent.getController().getName()=="Pot3-") {
      int val3 = int(cp5.getController("Pot3").getValue()) - 1;
      cp5.getController("Pot3").setValue(val3);
    }
    if (theEvent.getController().getName()=="Sw3") {
      int val3 = int(cp5.getController("Pot2").getValue());
      if (cp5.getController("Sw3").getValue() == 1) {
        sw3aVal = val3;
        sw3bVal = 0;
        if (debug)println("sw3aVal " + sw3aVal);
      }
      if (cp5.getController("Sw3").getValue() == 0) {
        sw3aVal = 0;
        sw3bVal = val3;
        if (debug)println("sw3bVal " + sw3bVal);
      }
    }


    if (theEvent.getController().getName()=="Pot4") {
      pot4Val = int(theEvent.getController().getValue());       
      if (debug) println("pot4Val " + pot4Val);
    }
    if (theEvent.getController().getName()=="Pot4+") {
      int val4 = int(cp5.getController("Pot4").getValue()) + 1;
      cp5.getController("Pot4").setValue(val4);
    }
    if (theEvent.getController().getName()=="Pot4-") {
      int val4 = int(cp5.getController("Pot4").getValue()) - 1;
      cp5.getController("Pot4").setValue(val4);
    }


    if (theEvent.getController().getName()=="Pot5") {
      int val5 = int(theEvent.getController().getValue());
      if (cp5.getController("Sw5").getValue() == 1) {
        pot5aVal = val5;
        pot5bVal = 0;
        if (debug) println("pot5aVal " + val5);
      }
      if (cp5.getController("Sw5").getValue() == 0) {
        pot5aVal = 0;
        pot5bVal = val5;
        if (debug) println("pot5bVal " + val5);
      }
    }
    if (theEvent.getController().getName()=="Pot5+") {
      int val5 = int(cp5.getController("Pot5").getValue()) + 1;
      cp5.getController("Pot5").setValue(val5);
    }
    if (theEvent.getController().getName()=="Pot5-") {
      int val5 = int(cp5.getController("Pot5").getValue()) - 1;
      cp5.getController("Pot5").setValue(val5);
    }
    if (theEvent.getController().getName()=="Sw5") {
      int val5 = int(cp5.getController("Pot5").getValue());
      if (cp5.getController("Sw5").getValue() == 1) {
        pot5aVal = val5;
        pot5bVal = 0;
        if (debug)println("pot5aVal " + pot5aVal);
      }
      if (cp5.getController("Sw5").getValue() == 0) {
        pot5aVal = 0;
        pot5bVal = val5;
        if (debug)println("pot5bVal " + pot5bVal);
      }
    }


    if (theEvent.getController().getName()=="LDR") {
      ldrVal = int(theEvent.getController().getValue());       
      if (debug) println("ldrVal " + ldrVal);
    }
    if (theEvent.getController().getName()=="LDR+") {
      int ldr = int(cp5.getController("LDR").getValue()) + 1;
      cp5.getController("LDR").setValue(ldr);
    }
    if (theEvent.getController().getName()=="LDR-") {
      int ldr = int(cp5.getController("LDR").getValue()) - 1;
      cp5.getController("LDR").setValue(ldr);
    }


    // Prints all the parameters
    if (theEvent.getController().getName()=="Write Preset") {
      if (debug) println("Write Preset");
      println(
        pot1Val + ", " + 
        sw1aVal + ", " + 
        sw1bVal + ", " + 
        sw2aVal + ", " + 
        sw2bVal + ", " + 
        sw3aVal + ", " + 
        sw3bVal + ", " + 
        pot4Val + ", " + 
        pot5aVal + ", " + 
        pot5bVal + ", " + 
        ldrVal       
        );
    }


    // Sets every parameter to zero / its initial state
    if (theEvent.getController().getName()=="Init") {
      if (debug) println("Init");
      cp5.getController("Pot1").setValue(0);
      cp5.getController("Pot2").setValue(0);
      cp5.getController("Sw1a").setValue(1);
      cp5.getController("Sw1b").setValue(0);
      cp5.getController("Sw2a").setValue(0);
      cp5.getController("Sw2b").setValue(0); 
      sw1sw2val = 0;
      cp5.getController("Pot3").setValue(0);
      cp5.getController("Sw3").setValue(1);
      cp5.getController("Pot4").setValue(0);
      cp5.getController("Pot5").setValue(0);
      cp5.getController("Sw5").setValue(1);
      cp5.getController("LDR").setValue(0);
    }


    // Randomizes every parameter
    if (theEvent.getController().getName()=="Random") {
      if (debug) println("Random");
      cp5.getController("Pot1").setValue(random(4096));
      float sw2rand = random(4);
      if (sw2rand>=0 && sw2rand <1) {
        cp5.getController("Sw1a").setValue(1);
        cp5.getController("Sw1b").setValue(0);
        cp5.getController("Sw2a").setValue(0);
        cp5.getController("Sw2b").setValue(0);
      }
      if (sw2rand>=1 && sw2rand <2) {
        cp5.getController("Sw1a").setValue(0);
        cp5.getController("Sw1b").setValue(1);
        cp5.getController("Sw2a").setValue(0);
        cp5.getController("Sw2b").setValue(0);
      }
      if (sw2rand>=2 && sw2rand <3) {
        cp5.getController("Sw1a").setValue(0);
        cp5.getController("Sw1b").setValue(0);
        cp5.getController("Sw2a").setValue(1);
        cp5.getController("Sw2b").setValue(0);
      }
      if (sw2rand>=3 && sw2rand <4) {
        cp5.getController("Sw1a").setValue(0);
        cp5.getController("Sw1b").setValue(0);
        cp5.getController("Sw2a").setValue(0);
        cp5.getController("Sw2b").setValue(1);
      }
      cp5.getController("Pot2").setValue(random(4096));
      if (random(1)>=0.5) cp5.getController("Sw3").setValue(0);
      else cp5.getController("Sw3").setValue(1);
      cp5.getController("Pot3").setValue(random(4096));
      cp5.getController("Pot4").setValue(random(4096));
      if (random(1)>=0.5) cp5.getController("Sw5").setValue(0);
      else cp5.getController("Sw5").setValue(1);
      cp5.getController("Pot5").setValue(random(4096));
      cp5.getController("LDR").setValue(random(4096));
    }
  }
}

// Sends the variables to the Arduino
void update() {
  //port.write("a" + pot1Val);
  //port.write("b" + sw1aVal);
  //port.write("c" + sw1bVal);
  //port.write("d" + sw2aVal);
  //port.write("e" + sw2bVal);
  //port.write("f" + sw3aVal);
  //port.write("g" + sw3bVal);
  //port.write("h" + pot4Val);
  //port.write("i" + pot5aVal);
  //port.write("j" + pot5bVal);
  //port.write("k" + ldrVal);
  
  port.write(pot1Val + ";" + sw1aVal + ";" + sw1bVal + ";" + sw2aVal + ";" + sw2bVal + ";" + sw3aVal + ";" +
             sw3bVal + ";" + pot4Val + ";" + pot5aVal + ";" + pot5bVal + ";" + ldrVal + ";");
}
